@@ -4,7 +4,7 @@ use std::ops::{Deref, DerefMut};
 use bitflags::bitflags;
 
 use ruff_index::{newtype_index, IndexSlice, IndexVec};
-use ruff_python_ast::call_path::format_call_path;
+use ruff_python_ast::name::QualifiedName;
 use ruff_python_ast::Stmt;
 use ruff_source_file::Locator;
 use ruff_text_size::{Ranged, TextRange};
@@ -110,7 +110,7 @@ impl<'a> Binding<'a> {
 
     /// Return `true` if this binding "redefines" the given binding, as per Pyflake's definition of
     /// redefinition.
-    pub fn redefines(&self, existing: &'a Binding) -> bool {
+    pub fn redefines(&self, existing: &Binding) -> bool {
         match &self.kind {
             // Submodule imports are only considered redefinitions if they import the same
             // submodule. For example, this is a redefinition:
@@ -125,38 +125,38 @@ impl<'a> Binding<'a> {
             // import foo.baz
             // ```
             BindingKind::Import(Import {
-                call_path: redefinition,
+                qualified_name: redefinition,
             }) => {
                 if let BindingKind::SubmoduleImport(SubmoduleImport {
-                    call_path: definition,
+                    qualified_name: definition,
                 }) = &existing.kind
                 {
                     return redefinition == definition;
                 }
             }
             BindingKind::FromImport(FromImport {
-                call_path: redefinition,
+                qualified_name: redefinition,
             }) => {
                 if let BindingKind::SubmoduleImport(SubmoduleImport {
-                    call_path: definition,
+                    qualified_name: definition,
                 }) = &existing.kind
                 {
                     return redefinition == definition;
                 }
             }
             BindingKind::SubmoduleImport(SubmoduleImport {
-                call_path: redefinition,
+                qualified_name: redefinition,
             }) => match &existing.kind {
                 BindingKind::Import(Import {
-                    call_path: definition,
+                    qualified_name: definition,
                 })
                 | BindingKind::SubmoduleImport(SubmoduleImport {
-                    call_path: definition,
+                    qualified_name: definition,
                 }) => {
                     return redefinition == definition;
                 }
                 BindingKind::FromImport(FromImport {
-                    call_path: definition,
+                    qualified_name: definition,
                 }) => {
                     return redefinition == definition;
                 }
@@ -184,12 +184,12 @@ impl<'a> Binding<'a> {
     }
 
     /// Returns the name of the binding (e.g., `x` in `x = 1`).
-    pub fn name<'b>(&self, locator: &'b Locator) -> &'b str {
+    pub fn name<'b>(&self, locator: &Locator<'b>) -> &'b str {
         locator.slice(self.range)
     }
 
     /// Returns the statement in which the binding was defined.
-    pub fn statement<'b>(&self, semantic: &'b SemanticModel) -> Option<&'b Stmt> {
+    pub fn statement<'b>(&self, semantic: &SemanticModel<'b>) -> Option<&'b Stmt> {
         self.source
             .map(|statement_id| semantic.statement(statement_id))
     }
@@ -205,7 +205,7 @@ impl<'a> Binding<'a> {
         })
     }
 
-    pub fn as_any_import(&'a self) -> Option<AnyImport<'a>> {
+    pub fn as_any_import(&self) -> Option<AnyImport<'_, 'a>> {
         match &self.kind {
             BindingKind::Import(import) => Some(AnyImport::Import(import)),
             BindingKind::SubmoduleImport(import) => Some(AnyImport::SubmoduleImport(import)),
@@ -362,7 +362,7 @@ pub struct Import<'a> {
     /// The full name of the module being imported.
     /// Ex) Given `import foo`, `qualified_name` would be "foo".
     /// Ex) Given `import foo as bar`, `qualified_name` would be "foo".
-    pub call_path: Box<[&'a str]>,
+    pub qualified_name: Box<QualifiedName<'a>>,
 }
 
 /// A binding for a member imported from a module, keyed on the name to which the member is bound.
@@ -373,7 +373,7 @@ pub struct FromImport<'a> {
     /// The full name of the member being imported.
     /// Ex) Given `from foo import bar`, `qualified_name` would be "foo.bar".
     /// Ex) Given `from foo import bar as baz`, `qualified_name` would be "foo.bar".
-    pub call_path: Box<[&'a str]>,
+    pub qualified_name: Box<QualifiedName<'a>>,
 }
 
 /// A binding for a submodule imported from a module, keyed on the name of the parent module.
@@ -382,7 +382,7 @@ pub struct FromImport<'a> {
 pub struct SubmoduleImport<'a> {
     /// The full name of the submodule being imported.
     /// Ex) Given `import foo.bar`, `qualified_name` would be "foo.bar".
-    pub call_path: Box<[&'a str]>,
+    pub qualified_name: Box<QualifiedName<'a>>,
 }
 
 #[derive(Debug, Clone, is_macro::Is)]
@@ -549,90 +549,85 @@ bitflags! {
 /// A trait for imported symbols.
 pub trait Imported<'a> {
     /// Returns the call path to the imported symbol.
-    fn call_path(&self) -> &[&str];
+    fn qualified_name(&self) -> &QualifiedName<'a>;
 
     /// Returns the module name of the imported symbol.
-    fn module_name(&self) -> &[&str];
+    fn module_name(&self) -> &[&'a str];
 
     /// Returns the member name of the imported symbol. For a straight import, this is equivalent
     /// to the qualified name; for a `from` import, this is the name of the imported symbol.
     fn member_name(&self) -> Cow<'a, str>;
-
-    /// Returns the fully-qualified name of the imported symbol.
-    fn qualified_name(&self) -> String {
-        format_call_path(self.call_path())
-    }
 }
 
 impl<'a> Imported<'a> for Import<'a> {
     /// For example, given `import foo`, returns `["foo"]`.
-    fn call_path(&self) -> &[&str] {
-        self.call_path.as_ref()
+    fn qualified_name(&self) -> &QualifiedName<'a> {
+        &self.qualified_name
     }
 
     /// For example, given `import foo`, returns `["foo"]`.
-    fn module_name(&self) -> &[&str] {
-        &self.call_path[..1]
+    fn module_name(&self) -> &[&'a str] {
+        &self.qualified_name.segments()[..1]
     }
 
     /// For example, given `import foo`, returns `"foo"`.
     fn member_name(&self) -> Cow<'a, str> {
-        Cow::Owned(self.qualified_name())
+        Cow::Owned(self.qualified_name().to_string())
     }
 }
 
 impl<'a> Imported<'a> for SubmoduleImport<'a> {
     /// For example, given `import foo.bar`, returns `["foo", "bar"]`.
-    fn call_path(&self) -> &[&str] {
-        self.call_path.as_ref()
+    fn qualified_name(&self) -> &QualifiedName<'a> {
+        &self.qualified_name
     }
 
     /// For example, given `import foo.bar`, returns `["foo"]`.
-    fn module_name(&self) -> &[&str] {
-        &self.call_path[..1]
+    fn module_name(&self) -> &[&'a str] {
+        &self.qualified_name.segments()[..1]
     }
 
     /// For example, given `import foo.bar`, returns `"foo.bar"`.
     fn member_name(&self) -> Cow<'a, str> {
-        Cow::Owned(self.qualified_name())
+        Cow::Owned(self.qualified_name().to_string())
     }
 }
 
 impl<'a> Imported<'a> for FromImport<'a> {
     /// For example, given `from foo import bar`, returns `["foo", "bar"]`.
-    fn call_path(&self) -> &[&str] {
-        self.call_path.as_ref()
+    fn qualified_name(&self) -> &QualifiedName<'a> {
+        &self.qualified_name
     }
 
     /// For example, given `from foo import bar`, returns `["foo"]`.
-    fn module_name(&self) -> &[&str] {
-        &self.call_path[..self.call_path.len() - 1]
+    fn module_name(&self) -> &[&'a str] {
+        &self.qualified_name.segments()[..self.qualified_name.segments().len() - 1]
     }
 
     /// For example, given `from foo import bar`, returns `"bar"`.
     fn member_name(&self) -> Cow<'a, str> {
-        Cow::Borrowed(self.call_path[self.call_path.len() - 1])
+        Cow::Borrowed(self.qualified_name.segments()[self.qualified_name.segments().len() - 1])
     }
 }
 
 /// A wrapper around an import [`BindingKind`] that can be any of the three types of imports.
 #[derive(Debug, Clone, is_macro::Is)]
-pub enum AnyImport<'a> {
-    Import(&'a Import<'a>),
-    SubmoduleImport(&'a SubmoduleImport<'a>),
-    FromImport(&'a FromImport<'a>),
+pub enum AnyImport<'a, 'ast> {
+    Import(&'a Import<'ast>),
+    SubmoduleImport(&'a SubmoduleImport<'ast>),
+    FromImport(&'a FromImport<'ast>),
 }
 
-impl<'a> Imported<'a> for AnyImport<'a> {
-    fn call_path(&self) -> &[&str] {
+impl<'a, 'ast> Imported<'ast> for AnyImport<'a, 'ast> {
+    fn qualified_name(&self) -> &QualifiedName<'ast> {
         match self {
-            Self::Import(import) => import.call_path(),
-            Self::SubmoduleImport(import) => import.call_path(),
-            Self::FromImport(import) => import.call_path(),
+            Self::Import(import) => import.qualified_name(),
+            Self::SubmoduleImport(import) => import.qualified_name(),
+            Self::FromImport(import) => import.qualified_name(),
         }
     }
 
-    fn module_name(&self) -> &[&str] {
+    fn module_name(&self) -> &[&'ast str] {
         match self {
             Self::Import(import) => import.module_name(),
             Self::SubmoduleImport(import) => import.module_name(),
@@ -640,7 +635,7 @@ impl<'a> Imported<'a> for AnyImport<'a> {
         }
     }
 
-    fn member_name(&self) -> Cow<'a, str> {
+    fn member_name(&self) -> Cow<'ast, str> {
         match self {
             Self::Import(import) => import.member_name(),
             Self::SubmoduleImport(import) => import.member_name(),
